@@ -16,10 +16,11 @@ from itertools import repeat
 import time
 from tqdm import tqdm
 
-from parsedmarc import IMAPError, get_dmarc_reports_from_inbox, \
-    parse_report_file, elastic, kafkaclient, splunk, save_output, \
-    watch_inbox, email_results, SMTPError, ParserError, __version__, \
-    InvalidDMARCReport
+from parsedmarc import get_dmarc_reports_from_inbox, watch_inbox, \
+    parse_report_file, get_dmarc_reports_from_mbox, elastic, kafkaclient, \
+    splunk, save_output, email_results, ParserError, __version__, \
+    InvalidDMARCReport, s3
+from parsedmarc.utils import is_mbox
 
 logger = logging.getLogger("parsedmarc")
 
@@ -30,10 +31,12 @@ def _str_to_list(s):
     return list(map(lambda i: i.lstrip(), _list))
 
 
-def cli_parse(file_path, sa, nameservers, dns_timeout, parallel=False):
+def cli_parse(file_path, sa, nameservers, dns_timeout, offline,
+              parallel=False):
     """Separated this function for multiprocessing"""
     try:
         file_results = parse_report_file(file_path,
+                                         offline=offline,
                                          nameservers=nameservers,
                                          dns_timeout=dns_timeout,
                                          strip_attachment_payloads=sa,
@@ -76,14 +79,27 @@ def _main():
                 )
             except Exception as error_:
                 logger.error("Kafka Error: {0}".format(error_.__str__()))
+        if opts.s3_bucket:
+            try:
+                s3_client = s3.S3Client(
+                    bucket_name=opts.s3_bucket,
+                    bucket_path=opts.s3_path,
+                )
+            except Exception as error_:
+                logger.error("S3 Error: {0}".format(error_.__str__()))
         if opts.save_aggregate:
             for report in reports_["aggregate_reports"]:
                 try:
                     if opts.elasticsearch_hosts:
+                        shards = opts.elasticsearch_number_of_shards
+                        replicas = opts.elasticsearch_number_of_replicas
                         elastic.save_aggregate_report_to_elasticsearch(
                             report,
                             index_suffix=opts.elasticsearch_index_suffix,
-                            monthly_indexes=opts.elasticsearch_monthly_indexes)
+                            monthly_indexes=opts.elasticsearch_monthly_indexes,
+                            number_of_shards=shards,
+                            number_of_replicas=replicas
+                        )
                 except elastic.AlreadySaved as warning:
                     logger.warning(warning.__str__())
                 except elastic.ElasticsearchError as error_:
@@ -96,6 +112,11 @@ def _main():
                 except Exception as error_:
                     logger.error("Kafka Error: {0}".format(
                          error_.__str__()))
+                try:
+                    if opts.s3_bucket:
+                        s3_client.save_aggregate_report_to_s3(report)
+                except Exception as error_:
+                    logger.error("S3 Error: {0}".format(error_.__str__()))
             if opts.hec:
                 try:
                     aggregate_reports_ = reports_["aggregate_reports"]
@@ -107,11 +128,15 @@ def _main():
         if opts.save_forensic:
             for report in reports_["forensic_reports"]:
                 try:
+                    shards = opts.elasticsearch_number_of_shards
+                    replicas = opts.elasticsearch_number_of_replicas
                     if opts.elasticsearch_hosts:
                         elastic.save_forensic_report_to_elasticsearch(
                             report,
                             index_suffix=opts.elasticsearch_index_suffix,
-                            monthly_indexes=opts.elasticsearch_monthly_indexes)
+                            monthly_indexes=opts.elasticsearch_monthly_indexes,
+                            number_of_shards=shards,
+                            number_of_replicas=replicas)
                 except elastic.AlreadySaved as warning:
                     logger.warning(warning.__str__())
                 except elastic.ElasticsearchError as error_:
@@ -126,6 +151,11 @@ def _main():
                 except Exception as error_:
                     logger.error("Kafka Error: {0}".format(
                         error_.__str__()))
+                try:
+                    if opts.s3_bucket:
+                        s3_client.save_forensic_report_to_s3(report)
+                except Exception as error_:
+                    logger.error("S3 Error: {0}".format(error_.__str__()))
             if opts.hec:
                 try:
                     forensic_reports_ = reports_["forensic_reports"]
@@ -137,27 +167,43 @@ def _main():
 
     arg_parser = ArgumentParser(description="Parses DMARC reports")
     arg_parser.add_argument("-c", "--config-file",
-                            help="A path to a configuration file "
+                            help="a path to a configuration file "
                                  "(--silent implied)")
     arg_parser.add_argument("file_path", nargs="*",
                             help="one or more paths to aggregate or forensic "
-                                 "report files or emails")
+                                 "report files, emails, or mbox files'")
     strip_attachment_help = "remove attachment payloads from forensic " \
                             "report output"
     arg_parser.add_argument("--strip-attachment-payloads",
                             help=strip_attachment_help, action="store_true")
     arg_parser.add_argument("-o", "--output",
                             help="write output files to the given directory")
+    arg_parser.add_argument("--aggregate-json-filename",
+                            help="filename for the aggregate JSON output file",
+                            default="aggregate.json")
+    arg_parser.add_argument("--forensic-json-filename",
+                            help="filename for the forensic JSON output file",
+                            default="forensic.json")
+    arg_parser.add_argument("--aggregate-csv-filename",
+                            help="filename for the aggregate CSV output file",
+                            default="aggregate.csv")
+    arg_parser.add_argument("--forensic-csv-filename",
+                            help="filename for the forensic CSV output file",
+                            default="forensic.csv")
     arg_parser.add_argument("-n", "--nameservers", nargs="+",
-                            help="nameservers to query "
-                                 "(default is Cloudflare's nameservers)")
+                            help="nameservers to query")
     arg_parser.add_argument("-t", "--dns_timeout",
                             help="number of seconds to wait for an answer "
-                                 "from DNS (default: 6.0)",
+                                 "from DNS (default: 2.0)",
                             type=float,
-                            default=6.0)
+                            default=2.0)
+    arg_parser.add_argument("--offline", action="store_true",
+                            help="do not make online queries for geolocation "
+                                 " or  DNS")
     arg_parser.add_argument("-s", "--silent", action="store_true",
                             help="only print errors and warnings")
+    arg_parser.add_argument("--verbose", action="store_true",
+                            help="more verbose output")
     arg_parser.add_argument("--debug", action="store_true",
                             help="print debugging information")
     arg_parser.add_argument("--log-file", default=None,
@@ -169,20 +215,29 @@ def _main():
     forensic_reports = []
 
     args = arg_parser.parse_args()
+
     opts = Namespace(file_path=args.file_path,
                      config_file=args.config_file,
+                     offline=args.offline,
                      strip_attachment_payloads=args.strip_attachment_payloads,
                      output=args.output,
+                     aggregate_csv_filename=args.aggregate_csv_filename,
+                     aggregate_json_filename=args.aggregate_json_filename,
+                     forensic_csv_filename=args.forensic_csv_filename,
+                     forensic_json_filename=args.forensic_json_filename,
                      nameservers=args.nameservers,
                      silent=args.silent,
                      dns_timeout=args.dns_timeout,
                      debug=args.debug,
+                     verbose=args.verbose,
                      save_aggregate=False,
                      save_forensic=False,
                      imap_host=None,
                      imap_skip_certificate_verification=False,
                      imap_ssl=True,
                      imap_port=993,
+                     imap_timeout=30,
+                     imap_max_retries=4,
                      imap_user=None,
                      imap_password=None,
                      imap_reports_folder="INBOX",
@@ -190,15 +245,21 @@ def _main():
                      imap_watch=False,
                      imap_delete=False,
                      imap_test=False,
+                     imap_batch_size=None,
                      hec=None,
                      hec_token=None,
                      hec_index=None,
                      hec_skip_certificate_verification=False,
                      elasticsearch_hosts=None,
+                     elasticsearch_timeout=60,
+                     elasticsearch_number_of_shards=1,
+                     elasticsearch_number_of_replicas=1,
                      elasticsearch_index_suffix=None,
                      elasticsearch_ssl=True,
                      elasticsearch_ssl_cert_path=None,
                      elasticsearch_monthly_indexes=False,
+                     elasticsearch_username=None,
+                     elasticsearch_password=None,
                      kafka_hosts=None,
                      kafka_username=None,
                      kafka_password=None,
@@ -216,6 +277,8 @@ def _main():
                      smtp_to=[],
                      smtp_subject="parsedmarc report",
                      smtp_message="Please see the attached DMARC results.",
+                     s3_bucket=None,
+                     s3_path=None,
                      log_file=args.log_file,
                      n_procs=1,
                      chunk_size=1
@@ -232,11 +295,25 @@ def _main():
         config.read(args.config_file)
         if "general" in config.sections():
             general_config = config["general"]
+            if "offline" in general_config:
+                opts.offline = general_config["offline"]
             if "strip_attachment_payloads" in general_config:
                 opts.strip_attachment_payloads = general_config[
                     "strip_attachment_payloads"]
             if "output" in general_config:
                 opts.output = general_config["output"]
+            if "aggregate_json_filename" in general_config:
+                opts.aggregate_json_filename = general_config[
+                    "aggregate_json_filename"]
+            if "forensic_json_filename" in general_config:
+                opts.forensic_json_filename = general_config[
+                    "forensic_json_filename"]
+            if "aggregate_csv_filename" in general_config:
+                opts.aggregate_csv_filename = general_config[
+                    "aggregate_csv_filename"]
+            if "forensic_csv_filename" in general_config:
+                opts.forensic_csv_filename = general_config[
+                    "forensic_csv_filename"]
             if "nameservers" in general_config:
                 opts.nameservers = _str_to_list(general_config["nameservers"])
             if "dns_timeout" in general_config:
@@ -247,6 +324,8 @@ def _main():
                 opts.save_forensic = general_config["save_forensic"]
             if "debug" in general_config:
                 opts.debug = general_config.getboolean("debug")
+            if "verbose" in general_config:
+                opts.verbose = general_config.getboolean("verbose")
             if "silent" in general_config:
                 opts.silent = general_config.getboolean("silent")
             if "log_file" in general_config:
@@ -264,7 +343,11 @@ def _main():
                              "imap config section")
                 exit(-1)
             if "port" in imap_config:
-                opts.imap_port = imap_config["port"]
+                opts.imap_port = imap_config.getint("port")
+            if "timeout" in imap_config:
+                opts.imap_timeout = imap_config.getfloat("timeout")
+            if "max_retries" in imap_config:
+                opts.imap_max_retries = imap_config.getint("max_retries")
             if "ssl" in imap_config:
                 opts.imap_ssl = imap_config.getboolean("ssl")
             if "skip_certificate_verification" in imap_config:
@@ -294,6 +377,10 @@ def _main():
                 opts.imap_delete = imap_config.getboolean("delete")
             if "test" in imap_config:
                 opts.imap_test = imap_config.getboolean("test")
+            if "batch_size" in imap_config:
+                opts.imap_batch_size = imap_config.getint("batch_size")
+            else:
+                opts.imap_batch_size = None
         if "elasticsearch" in config:
             elasticsearch_config = config["elasticsearch"]
             if "hosts" in elasticsearch_config:
@@ -303,6 +390,17 @@ def _main():
                 logger.critical("hosts setting missing from the "
                                 "elasticsearch config section")
                 exit(-1)
+            if "timeout" in elasticsearch_config:
+                timeout = elasticsearch_config.getfloat("timeout")
+                opts.elasticsearch_timeout = timeout
+            if "number_of_shards" in elasticsearch_config:
+                number_of_shards = elasticsearch_config.getint(
+                    "number_of_shards")
+                opts.elasticsearch_number_of_shards = number_of_shards
+                if "number_of_replicas" in elasticsearch_config:
+                    number_of_replicas = elasticsearch_config.getint(
+                        "number_of_replicas")
+                    opts.elasticsearch_number_of_replicas = number_of_replicas
             if "index_suffix" in elasticsearch_config:
                 opts.elasticsearch_index_suffix = elasticsearch_config[
                     "index_suffix"]
@@ -315,6 +413,12 @@ def _main():
             if "cert_path" in elasticsearch_config:
                 opts.elasticsearch_ssl_cert_path = elasticsearch_config[
                     "cert_path"]
+            if "user" in elasticsearch_config:
+                opts.elasticsearch_username = elasticsearch_config[
+                    "user"]
+            if "password" in elasticsearch_config:
+                opts.elasticsearch_password = elasticsearch_config[
+                    "password"]
         if "splunk_hec" in config.sections():
             hec_config = config["splunk_hec"]
             if "url" in hec_config:
@@ -419,10 +523,29 @@ def _main():
                 opts.smtp_attachment = smtp_config["attachment"]
             if "message" in smtp_config:
                 opts.smtp_message = smtp_config["message"]
+        if "s3" in config.sections():
+            s3_config = config["s3"]
+            if "bucket" in s3_config:
+                opts.s3_bucket = s3_config["bucket"]
+            else:
+                logger.critical("bucket setting missing from the "
+                                "s3 config section")
+                exit(-1)
+            if "path" in s3_config:
+                opts.s3_path = s3_config["path"]
+                if opts.s3_path.startswith("/"):
+                    opts.s3_path = opts.s3_path[1:]
+                if opts.s3_path.endswith("/"):
+                    opts.s3_path = opts.s3_path[:-1]
+            else:
+                opts.s3_path = ""
 
     logging.basicConfig(level=logging.WARNING)
     logger.setLevel(logging.WARNING)
 
+    if opts.verbose:
+        logging.basicConfig(level=logging.INFO)
+        logger.setLevel(logging.INFO)
     if opts.debug:
         logging.basicConfig(level=logging.DEBUG)
         logger.setLevel(logging.DEBUG)
@@ -437,6 +560,8 @@ def _main():
         logger.error("You must supply input files, or an IMAP configuration")
         exit(1)
 
+    logger.info("Starting dmarcparse")
+
     if opts.save_aggregate or opts.save_forensic:
         try:
             if opts.elasticsearch_hosts:
@@ -450,7 +575,10 @@ def _main():
                         es_forensic_index, suffix)
                 elastic.set_hosts(opts.elasticsearch_hosts,
                                   opts.elasticsearch_ssl,
-                                  opts.elasticsearch_ssl_cert_path)
+                                  opts.elasticsearch_ssl_cert_path,
+                                  opts.elasticsearch_username,
+                                  opts.elasticsearch_password,
+                                  timeout=opts.elasticsearch_timeout)
                 elastic.migrate_indexes(aggregate_indexes=[es_aggregate_index],
                                         forensic_indexes=[es_forensic_index])
         except elastic.ElasticsearchError as error:
@@ -474,9 +602,19 @@ def _main():
     kafka_forensic_topic = opts.kafka_forensic_topic
 
     file_paths = []
+    mbox_paths = []
+
     for file_path in args.file_path:
         file_paths += glob(file_path)
+    for file_path in file_paths:
+        if is_mbox(file_path):
+            mbox_paths.append(file_path)
+
     file_paths = list(set(file_paths))
+    mbox_paths = list(set(mbox_paths))
+
+    for mbox_path in mbox_paths:
+        file_paths.remove(mbox_path)
 
     counter = Value('i', 0)
     pool = Pool(opts.n_procs, initializer=init, initargs=(counter,))
@@ -485,6 +623,7 @@ def _main():
                                      repeat(opts.strip_attachment_payloads),
                                      repeat(opts.nameservers),
                                      repeat(opts.dns_timeout),
+                                     repeat(opts.offline),
                                      repeat(opts.n_procs >= 1)),
                                  opts.chunk_size)
     pbar = tqdm(total=len(file_paths))
@@ -506,6 +645,14 @@ def _main():
             elif result[0]["report_type"] == "forensic":
                 forensic_reports.append(result[0]["report"])
 
+    for mbox_path in mbox_paths:
+        reports = get_dmarc_reports_from_mbox(mbox_path, opts.nameservers,
+                                              opts.dns_timeout,
+                                              opts.strip_attachment_payloads,
+                                              opts.offline, False)
+        aggregate_reports += reports["aggregate_reports"]
+        forensic_reports += reports["forensic_reports"]
+
     if opts.imap_host:
         try:
             if opts.imap_user is None or opts.imap_password is None:
@@ -517,32 +664,35 @@ def _main():
             ns = opts.nameservers
             sa = opts.strip_attachment_payloads
             ssl = True
-            ssl_context = None
+            verify = True
             if opts.imap_skip_certificate_verification:
                 logger.debug("Skipping IMAP certificate verification")
-                ssl_context = create_default_context()
-                ssl_context.check_hostname = False
-                ssl_context.verify_mode = CERT_NONE
+                verify = False
             if opts.imap_ssl is False:
                 ssl = False
-            reports = get_dmarc_reports_from_inbox(host=opts.imap_host,
-                                                   port=opts.imap_port,
-                                                   ssl=ssl,
-                                                   ssl_context=ssl_context,
-                                                   user=opts.imap_user,
-                                                   password=opts.imap_password,
-                                                   reports_folder=rf,
-                                                   archive_folder=af,
-                                                   delete=opts.imap_delete,
-                                                   nameservers=ns,
-                                                   test=opts.imap_test,
-                                                   strip_attachment_payloads=sa
-                                                   )
+            reports = get_dmarc_reports_from_inbox(
+                host=opts.imap_host,
+                port=opts.imap_port,
+                ssl=ssl,
+                verify=verify,
+                timeout=opts.imap_timeout,
+                max_retries=opts.imap_max_retries,
+                user=opts.imap_user,
+                password=opts.imap_password,
+                reports_folder=rf,
+                archive_folder=af,
+                delete=opts.imap_delete,
+                offline=opts.offline,
+                nameservers=ns,
+                test=opts.imap_test,
+                strip_attachment_payloads=sa,
+                batch_size=opts.imap_batch_size
+            )
 
             aggregate_reports += reports["aggregate_reports"]
             forensic_reports += reports["forensic_reports"]
 
-        except IMAPError as error:
+        except Exception as error:
             logger.error("IMAP Error: {0}".format(error.__str__()))
             exit(1)
 
@@ -550,52 +700,58 @@ def _main():
                            ("forensic_reports", forensic_reports)])
 
     if opts.output:
-        save_output(results, output_directory=opts.output)
+        save_output(results, output_directory=opts.output,
+                    aggregate_json_filename=opts.aggregate_json_filename,
+                    forensic_json_filename=opts.forensic_json_filename,
+                    aggregate_csv_filename=opts.aggregate_csv_filename,
+                    forensic_csv_filename=opts.forensic_csv_filename)
 
     process_reports(results)
 
     if opts.smtp_host:
         try:
-            ssl_context = None
+            verify = True
             if opts.smtp_skip_certificate_verification:
-                logger.debug("Skipping SMTP certificate verification")
-                ssl_context = create_default_context()
-                ssl_context.check_hostname = False
-                ssl_context.verify_mode = CERT_NONE
+                verify = False
             email_results(results, opts.smtp_host, opts.smtp_from,
-                          opts.smtp_to, ssl=opts.smtp_ssl,
-                          user=opts.smtp_user,
+                          opts.smtp_to, port=opts.smtp_port, verify=verify,
+                          username=opts.smtp_user,
                           password=opts.smtp_password,
-                          subject=opts.smtp_subject,
-                          ssl_context=ssl_context)
-        except SMTPError as error:
-            logger.error("SMTP Error: {0}".format(error.__str__()))
+                          subject=opts.smtp_subject)
+        except Exception as error:
+            logger.error("{0}".format(error.__str__()))
             exit(1)
 
     if opts.imap_host and opts.imap_watch:
         logger.info("Watching for email - Quit with ctrl-c")
         ssl = True
-        ssl_context = None
+        verify = True
         if opts.imap_skip_certificate_verification:
             logger.debug("Skipping IMAP certificate verification")
-            ssl_context = create_default_context()
-            ssl_context.check_hostname = False
-            ssl_context.verify_mode = CERT_NONE
+            verify = False
         if opts.imap_ssl is False:
             ssl = False
         try:
             sa = opts.strip_attachment_payloads
-            watch_inbox(opts.imap_host, opts.imap_user, opts.imap_password,
-                        process_reports, port=opts.imap_port, ssl=ssl,
-                        ssl_context=ssl_context,
-                        reports_folder=opts.imap_reports_folder,
-                        archive_folder=opts.imap_archive_folder,
-                        delete=opts.imap_delete,
-                        test=opts.imap_test, nameservers=opts.nameservers,
-                        dns_timeout=opts.dns_timeout,
-                        strip_attachment_payloads=sa)
-        except IMAPError as error:
-            logger.error("IMAP error: {0}".format(error.__str__()))
+            watch_inbox(
+                opts.imap_host,
+                opts.imap_user,
+                opts.imap_password,
+                process_reports,
+                port=opts.imap_port,
+                ssl=ssl,
+                verify=verify,
+                reports_folder=opts.imap_reports_folder,
+                archive_folder=opts.imap_archive_folder,
+                delete=opts.imap_delete,
+                test=opts.imap_test,
+                nameservers=opts.nameservers,
+                dns_timeout=opts.dns_timeout,
+                strip_attachment_payloads=sa,
+                batch_size=opts.imap_batch_size,
+                offline=opts.offline)
+        except FileExistsError as error:
+            logger.error("{0}".format(error.__str__()))
             exit(1)
 
 
