@@ -56,6 +56,8 @@ class _AggregateReportDoc(Document):
     org_extra_contact_info = Text()
     report_id = Text()
     date_range = Date()
+    date_begin = Date()
+    date_end = Date()
     errors = Text()
     published_policy = Object(_PublishedPolicy)
     source_ip_address = Ip()
@@ -169,7 +171,8 @@ class AlreadySaved(ValueError):
     """Raised when a report to be saved matches an existing report"""
 
 
-def set_hosts(hosts, use_ssl=False, ssl_cert_path=None):
+def set_hosts(hosts, use_ssl=False, ssl_cert_path=None,
+              username=None, password=None, timeout=60.0):
     """
     Sets the Elasticsearch hosts to use
 
@@ -177,12 +180,15 @@ def set_hosts(hosts, use_ssl=False, ssl_cert_path=None):
         hosts (str): A single hostname or URL, or list of hostnames or URLs
         use_ssl (bool): Use a HTTPS connection to the server
         ssl_cert_path (str): Path to the certificate chain
+        username (str): The username to use for authentication
+        password (str): The password to use for authentication
+        timeout (float): Timeout in seconds
     """
     if type(hosts) != list:
         hosts = [hosts]
     conn_params = {
         "hosts": hosts,
-        "timeout": 20
+        "timeout": timeout
     }
     if use_ssl:
         conn_params['use_ssl'] = True
@@ -191,6 +197,8 @@ def set_hosts(hosts, use_ssl=False, ssl_cert_path=None):
             conn_params['ca_certs'] = ssl_cert_path
         else:
             conn_params['verify_certs'] = False
+    if username:
+        conn_params['http_auth'] = (username+":"+password)
     connections.create_connection(**conn_params)
 
 
@@ -209,9 +217,10 @@ def create_indexes(names, settings=None):
             if not index.exists():
                 logger.debug("Creating Elasticsearch index: {0}".format(name))
                 if settings is None:
-                    settings = dict(number_of_shards=1,
-                                    number_of_replicas=1)
-                index.put_settings(settings)
+                    index.settings(number_of_shards=1,
+                                   number_of_replicas=1)
+                else:
+                    index.settings(**settings)
                 index.create()
         except Exception as e:
             raise ElasticsearchError(
@@ -270,7 +279,9 @@ def migrate_indexes(aggregate_indexes=None, forensic_indexes=None):
 
 def save_aggregate_report_to_elasticsearch(aggregate_report,
                                            index_suffix=None,
-                                           monthly_indexes=False):
+                                           monthly_indexes=False,
+                                           number_of_shards=1,
+                                           number_of_replicas=1):
     """
     Saves a parsed DMARC aggregate report to ElasticSearch
 
@@ -278,20 +289,24 @@ def save_aggregate_report_to_elasticsearch(aggregate_report,
         aggregate_report (OrderedDict): A parsed forensic report
         index_suffix (str): The suffix of the name of the index to save to
         monthly_indexes (bool): Use monthly indexes instead of daily indexes
+        number_of_shards (int): The number of shards to use in the index
+        number_of_replicas (int): The number of replicas to use in the index
 
     Raises:
             AlreadySaved
     """
-    logger.debug("Saving aggregate report to Elasticsearch")
+    logger.info("Saving aggregate report to Elasticsearch")
     aggregate_report = aggregate_report.copy()
     metadata = aggregate_report["report_metadata"]
     org_name = metadata["org_name"]
     report_id = metadata["report_id"]
     domain = aggregate_report["policy_published"]["domain"]
-    begin_date = human_timestamp_to_datetime(metadata["begin_date"])
-    end_date = human_timestamp_to_datetime(metadata["end_date"])
-    begin_date_human = begin_date.strftime("%Y-%m-%d %H:%M:%S")
-    end_date_human = end_date.strftime("%Y-%m-%d %H:%M:%S")
+    begin_date = human_timestamp_to_datetime(metadata["begin_date"],
+                                             to_utc=True)
+    end_date = human_timestamp_to_datetime(metadata["end_date"],
+                                           to_utc=True)
+    begin_date_human = begin_date.strftime("%Y-%m-%d %H:%M:%SZ")
+    end_date_human = end_date.strftime("%Y-%m-%d %H:%M:%SZ")
     if monthly_indexes:
         index_date = begin_date.strftime("%Y-%m")
     else:
@@ -301,13 +316,16 @@ def save_aggregate_report_to_elasticsearch(aggregate_report,
     date_range = [aggregate_report["begin_date"],
                   aggregate_report["end_date"]]
 
-    org_name_query = Q(dict(match=dict(org_name=org_name)))
-    report_id_query = Q(dict(match=dict(report_id=report_id)))
-    domain_query = Q(dict(match={"published_policy.domain": domain}))
-    begin_date_query = Q(dict(match=dict(date_range=begin_date)))
-    end_date_query = Q(dict(match=dict(date_range=end_date)))
+    org_name_query = Q(dict(match_phrase=dict(org_name=org_name)))
+    report_id_query = Q(dict(match_phrase=dict(report_id=report_id)))
+    domain_query = Q(dict(match_phrase={"published_policy.domain": domain}))
+    begin_date_query = Q(dict(match=dict(date_begin=begin_date)))
+    end_date_query = Q(dict(match=dict(date_end=end_date)))
 
-    search = Search(index="dmarc_aggregate*")
+    if index_suffix is not None:
+        search = Search(index="dmarc_aggregate_{0}*".format(index_suffix))
+    else:
+        search = Search(index="dmarc_aggregate*")
     query = org_name_query & report_id_query & domain_query
     query = query & begin_date_query & end_date_query
     search.query = query
@@ -334,12 +352,14 @@ def save_aggregate_report_to_elasticsearch(aggregate_report,
 
     for record in aggregate_report["records"]:
         agg_doc = _AggregateReportDoc(
-            xml_schemea=aggregate_report["xml_schema"],
+            xml_schema=aggregate_report["xml_schema"],
             org_name=metadata["org_name"],
             org_email=metadata["org_email"],
             org_extra_contact_info=metadata["org_extra_contact_info"],
             report_id=metadata["report_id"],
             date_range=date_range,
+            date_begin=aggregate_report["begin_date"],
+            date_end=aggregate_report["end_date"],
             errors=metadata["errors"],
             published_policy=published_policy,
             source_ip_address=record["source"]["ip_address"],
@@ -348,8 +368,10 @@ def save_aggregate_report_to_elasticsearch(aggregate_report,
             source_base_domain=record["source"]["base_domain"],
             message_count=record["count"],
             disposition=record["policy_evaluated"]["disposition"],
-            dkim_aligned=record["policy_evaluated"]["dkim"] == "pass",
-            spf_aligned=record["policy_evaluated"]["spf"] == "pass",
+            dkim_aligned=record["policy_evaluated"]["dkim"] is not None and
+            record["policy_evaluated"]["dkim"].lower() == "pass",
+            spf_aligned=record["policy_evaluated"]["spf"] is not None and
+            record["policy_evaluated"]["spf"].lower() == "pass",
             header_from=record["identifiers"]["header_from"],
             envelope_from=record["identifiers"]["envelope_from"],
             envelope_to=record["identifiers"]["envelope_to"]
@@ -373,7 +395,9 @@ def save_aggregate_report_to_elasticsearch(aggregate_report,
         if index_suffix:
             index = "{0}_{1}".format(index, index_suffix)
         index = "{0}-{1}".format(index, index_date)
-        create_indexes([index])
+        index_settings = dict(number_of_shards=number_of_shards,
+                              number_of_replicas=number_of_replicas)
+        create_indexes([index], index_settings)
         agg_doc.meta.index = index
 
         try:
@@ -385,7 +409,9 @@ def save_aggregate_report_to_elasticsearch(aggregate_report,
 
 def save_forensic_report_to_elasticsearch(forensic_report,
                                           index_suffix=None,
-                                          monthly_indexes=False):
+                                          monthly_indexes=False,
+                                          number_of_shards=1,
+                                          number_of_replicas=1):
     """
         Saves a parsed DMARC forensic report to ElasticSearch
 
@@ -394,12 +420,15 @@ def save_forensic_report_to_elasticsearch(forensic_report,
             index_suffix (str): The suffix of the name of the index to save to
             monthly_indexes (bool): Use monthly indexes instead of daily
                                     indexes
+            number_of_shards (int): The number of shards to use in the index
+            number_of_replicas (int): The number of replicas to use in the
+                                      index
 
         Raises:
             AlreadySaved
 
         """
-    logger.debug("Saving forensic report to Elasticsearch")
+    logger.info("Saving forensic report to Elasticsearch")
     forensic_report = forensic_report.copy()
     sample_date = None
     if forensic_report["parsed_sample"]["date"] is not None:
@@ -413,7 +442,10 @@ def save_forensic_report_to_elasticsearch(forensic_report,
     arrival_date_human = forensic_report["arrival_date_utc"]
     arrival_date = human_timestamp_to_datetime(arrival_date_human)
 
-    search = Search(index="dmarc_forensic*")
+    if index_suffix is not None:
+        search = Search(index="dmarc_forensic_{0}*".format(index_suffix))
+    else:
+        search = Search(index="dmarc_forensic*")
     arrival_query = {"match": {"arrival_date": arrival_date}}
     q = Q(arrival_query)
 
@@ -422,15 +454,15 @@ def save_forensic_report_to_elasticsearch(forensic_report,
     subject = None
     if "from" in headers:
         from_ = headers["from"]
-        from_query = {"match": {"sample.headers.from": from_}}
+        from_query = {"match_phrase": {"sample.headers.from": from_}}
         q = q & Q(from_query)
     if "to" in headers:
         to_ = headers["to"]
-        to_query = {"match": {"sample.headers.to": to_}}
+        to_query = {"match_phrase": {"sample.headers.to": to_}}
         q = q & Q(to_query)
     if "subject" in headers:
         subject = headers["subject"]
-        subject_query = {"match": {"sample.headers.subject": subject}}
+        subject_query = {"match_phrase": {"sample.headers.subject": subject}}
         q = q & Q(subject_query)
 
     search.query = q
@@ -504,7 +536,9 @@ def save_forensic_report_to_elasticsearch(forensic_report,
         else:
             index_date = arrival_date.strftime("%Y-%m-%d")
         index = "{0}-{1}".format(index, index_date)
-        create_indexes([index])
+        index_settings = dict(number_of_shards=number_of_shards,
+                              number_of_replicas=number_of_replicas)
+        create_indexes([index], index_settings)
         forensic_doc.meta.index = index
         try:
             forensic_doc.save()
