@@ -5,8 +5,6 @@ import os
 from datetime import datetime
 from datetime import timedelta
 from collections import OrderedDict
-from io import BytesIO
-import tarfile
 import tempfile
 import subprocess
 import shutil
@@ -16,6 +14,8 @@ import hashlib
 import base64
 import platform
 import atexit
+import mailbox
+import re
 
 import dateparser
 import dns.reversename
@@ -26,11 +26,13 @@ import geoip2.errors
 import requests
 import publicsuffix2
 
-USER_AGENT = "Mozilla/5.0 ((0 {1})) parsedmarc".format(
+USER_AGENT = "Mozilla/5.0 (({0} {1})) parsedmarc".format(
             platform.system(),
             platform.release(),
         )
 
+parenthesis_regex = re.compile(r'\s*\(.*\)\s*')
+
 null_file = open(os.devnull, "w")
 logger = logging.getLogger("parsedmarc")
 mailparser_logger = logging.getLogger("mailparser")
@@ -51,6 +53,10 @@ class EmailParserError(RuntimeError):
     """Raised when an error parsing the email occurs"""
 
 
+class DownloadError(RuntimeError):
+    """Rasied when an error occurs when downloading a file"""
+
+
 def decode_base64(data):
     """
     Decodes a base64 string, with padding being optional
@@ -91,9 +97,13 @@ def get_base_domain(domain, use_fresh_psl=False):
         url = "https://publicsuffix.org/list/public_suffix_list.dat"
         # Use a browser-like user agent string to bypass some proxy blocks
         headers = {"User-Agent": USER_AGENT}
-        fresh_psl = requests.get(url, headers=headers).text
-        with open(psl_path, "w", encoding="utf-8") as fresh_psl_file:
-            fresh_psl_file.write(fresh_psl)
+        try:
+            fresh_psl = requests.get(url, headers=headers).text
+            with open(psl_path, "w", encoding="utf-8") as fresh_psl_file:
+                fresh_psl_file.write(fresh_psl)
+        except Exception as error:
+            raise DownloadError(
+                "Failed to download an updated PSL {0}".format(error))
 
     if use_fresh_psl:
         if not os.path.exists(psl_path):
@@ -102,17 +112,14 @@ def get_base_domain(domain, use_fresh_psl=False):
             psl_age = datetime.now() - datetime.fromtimestamp(
                 os.stat(psl_path).st_mtime)
             if psl_age > timedelta(hours=24):
-                try:
-                    download_psl()
-                except Exception as error:
-                    logger.warning(
-                        "Failed to download an updated PSL {0}".format(error))
+                download_psl()
+
         with open(psl_path, encoding="utf-8") as psl_file:
             psl = publicsuffix2.PublicSuffixList(psl_file)
 
         return psl.get_public_suffix(domain)
     else:
-        return publicsuffix2.get_public_suffix(domain)
+        return publicsuffix2.get_sld(domain)
 
 
 def query_dns(domain, record_type, cache=None, nameservers=None, timeout=2.0):
@@ -150,7 +157,7 @@ def query_dns(domain, record_type, cache=None, nameservers=None, timeout=2.0):
     if record_type == "TXT":
         resource_records = list(map(
             lambda r: r.strings,
-            resolver.query(domain, record_type, tcp=True)))
+            resolver.resolve(domain, record_type, lifetime=timeout)))
         _resource_record = [
             resource_record[0][:0].join(resource_record)
             for resource_record in resource_records if resource_record]
@@ -158,7 +165,7 @@ def query_dns(domain, record_type, cache=None, nameservers=None, timeout=2.0):
     else:
         records = list(map(
             lambda r: r.to_text().replace('"', '').rstrip("."),
-            resolver.query(domain, record_type, tcp=True)))
+            resolver.resolve(domain, record_type, lifetime=timeout)))
     if cache:
         cache[cache_key] = records
 
@@ -230,6 +237,8 @@ def human_timestamp_to_datetime(human_timestamp, to_utc=False):
         DateTime: The converted timestamp
     """
 
+    human_timestamp = human_timestamp.replace("-0000", "")
+    human_timestamp = parenthesis_regex.sub("", human_timestamp)
     settings = {}
 
     if to_utc:
@@ -252,52 +261,25 @@ def human_timestamp_to_timestamp(human_timestamp):
     return human_timestamp_to_datetime(human_timestamp).timestamp()
 
 
-def get_ip_address_country(ip_address, parallel=False):
+def get_ip_address_country(ip_address):
     """
     Uses the MaxMind Geolite2 Country database to return the ISO code for the
     country associated with the given IPv4 or IPv6 address
 
     Args:
         ip_address (str): The IP address to query for
-        parallel (bool): Parallel processing
 
     Returns:
         str: And ISO country code associated with the given IP address
     """
-    def download_country_database(location="GeoLite2-Country.mmdb"):
-        """Downloads the MaxMind Geolite2 Country database
-
-        Args:
-            location (str): Local location for the database file
-        """
-        if parallel:
-            logging.warning("Cannot download GeoIP database in parallel mode")
-            return
-        url = "https://geolite.maxmind.com/download/geoip/database/" \
-              "GeoLite2-Country.tar.gz"
-        # Use a browser-like user agent string to bypass some proxy blocks
-        headers = {"User-Agent": USER_AGENT}
-        original_filename = "GeoLite2-Country.mmdb"
-        try:
-            response = requests.get(url, headers=headers)
-            response.raise_for_status()
-            tar_bytes = response.content
-            tar_file = tarfile.open(fileobj=BytesIO(tar_bytes), mode="r:gz")
-            tar_dir = tar_file.getnames()[0]
-            tar_path = "{0}/{1}".format(tar_dir, original_filename)
-            tar_file.extract(tar_path)
-            shutil.move(tar_path, location)
-            shutil.rmtree(tar_dir)
-        except Exception as e:
-            logger.warning("Error downloading {0}: {1}".format(url,
-                                                               e.__str__()))
-
     system_paths = [
         "GeoLite2-Country.mmdb",
         "/usr/local/share/GeoIP/GeoLite2-Country.mmdb",
         "/usr/share/GeoIP/GeoLite2-Country.mmdb",
         "/var/lib/GeoIP/GeoLite2-Country.mmdb",
         "/var/local/lib/GeoIP/GeoLite2-Country.mmdb",
+        "%SystemDrive%\\ProgramData\\MaxMind\\GeoIPUpdate\\GeoIP\\"
+        "GeoLite2-Country.mmdb",
         "C:\\GeoIP\\GeoLite2-Country.mmdb"
     ]
 
@@ -311,14 +293,16 @@ def get_ip_address_country(ip_address, parallel=False):
     if db_path is None:
         db_path = os.path.join(tempdir, "GeoLite2-Country.mmdb")
         if not os.path.exists(db_path):
-            download_country_database(db_path)
-            if not os.path.exists(db_path):
-                return None
+            logging.warning("GeoLite2-Country.mmdb is missing. "
+                            "Please follow the instructions at "
+                            "https://dev.maxmind.com/geoip/geoipupdate/ "
+                            "to get the latest version.")
+            return None
         else:
             db_age = datetime.now() - datetime.fromtimestamp(
                 os.stat(db_path).st_mtime)
             if db_age > timedelta(days=7):
-                download_country_database()
+                logger.warning("GeoLite2-Country.mmdb is more than a week old")
         db_path = db_path
 
     db_reader = geoip2.database.Reader(db_path)
@@ -333,14 +317,15 @@ def get_ip_address_country(ip_address, parallel=False):
     return country
 
 
-def get_ip_address_info(ip_address, cache=None, nameservers=None,
-                        timeout=2.0, parallel=False):
+def get_ip_address_info(ip_address, cache=None, offline=False,
+                        nameservers=None, timeout=2.0, parallel=False):
     """
     Returns reverse DNS and country information for the given IP address
 
     Args:
         ip_address (str): The IP address to check
         cache (ExpiringDict): Cache storage
+        offline (bool): Do not make online queries for geolocation or DNS
         nameservers (list): A list of one or more nameservers to use
         (Cloudflare's public DNS resolvers by default)
         timeout (float): Sets the DNS timeout in seconds
@@ -357,10 +342,13 @@ def get_ip_address_info(ip_address, cache=None, nameservers=None,
             return info
     info = OrderedDict()
     info["ip_address"] = ip_address
-    reverse_dns = get_reverse_dns(ip_address,
-                                  nameservers=nameservers,
-                                  timeout=timeout)
-    country = get_ip_address_country(ip_address, parallel=parallel)
+    if offline:
+        reverse_dns = None
+    else:
+        reverse_dns = get_reverse_dns(ip_address,
+                                      nameservers=nameservers,
+                                      timeout=timeout)
+    country = get_ip_address_country(ip_address)
     info["country"] = country
     info["reverse_dns"] = reverse_dns
     info["base_domain"] = None
@@ -407,9 +395,32 @@ def get_filename_safe_string(string):
         string = string.replace(char, "")
     string = string.rstrip(".")
 
+    string = (string[:100]) if len(string) > 100 else string
+
     return string
 
 
+def is_mbox(path):
+    """
+    Checks if the given content is a MBOX mailbox file
+
+    Args:
+        path: Content to check
+
+    Returns:
+        bool: A flag the indicates if a file is a MBOX mailbox file
+    """
+    _is_mbox = False
+    try:
+        mbox = mailbox.mbox(path)
+        if len(mbox.keys()) > 0:
+            _is_mbox = True
+    except Exception as e:
+        logger.debug("Error checking for MBOX file: {0}".format(e.__str__()))
+
+    return _is_mbox
+
+
 def is_outlook_msg(content):
     """
     Checks if the given content is a Outlook msg OLE file
